"""
========================================
  Introduction to Computer Programming
========================================

----------------------------------------------------
  examples in preparation for lab 5:
    * Read Evaluate Print Loops (REPLs)
    * input function
    * converting strings to integers with int
    * formatting numbers into strings with f-string literals
    * break out of loops
    * None is None
    * functions as values
    * top-down style
----------------------------------------------------
"""


from random import randrange

from simple_graphics import clear, paint, COLUMNS, ROWS


# ------------------- using input to get text from user --------------------

# purely side effecting: takes no argument, does not return value
def hello():
    name = input('Enter your name: ')
    print('Hello, ' + name)


# purely side effecting: takes no argument, does not return value
def greet():
    name = input('Enter your name: ')
    if name == 'Mike':
        print('Oh, you again.')
    else:
        print('Greetings, ' + name + '!')


# takes no argument, but does return value
def generate_length():
    name = input('Enter some text: ')
    n = len(name)
    return n


def string_repl():
    s = ''
    t = input('enter a line of text: ')
    while len(t) > 0:
        s = s + t + '\n'
        t = input('enter another line of text: ')
    print('here are the lines you entered:\n')
    print(s)


# ----------------- converting between strings and integers ----------------

# using int, input to generate integers
def get_seconds():
    minutes = int(input('Enter a number of minutes: '))
    seconds = minutes * 60
    return seconds


# using "f strings" to combine strings and integers in printed output
def show_time(s):
    m = s // 60
    r = s % 60
    print(f'{s} total seconds is really {m} minutes and {r} seconds.')


# combining both approaches:
def total_seconds():
    m = int(input('Enter a number of minutes: '))
    s = m * 60
    print(f'total number of seconds in {m} minutes is {s}.')


# using both in a simple REPL to add numbers
# observe how f strings work with {x=}
def repl_sum():
    sum = 0
    n = int(input('enter a number: '))
    while n != 0:
        sum += n
        n = int(input('enter another number: '))
    print(f'final {sum = }')


# ----------------------- top-down design example ------------------------

def top_down_guessing_game():
    welcome()
    n = randrange(1, 10)
    game_loop(n)
    byebye()


def game_loop(hidden_number):
    count = 0
    guess = None
    while guess != hidden_number:
        print(f'you have made {count} guesses')
        guess = int(input('make a guess: '))
        count += 1
    print(f'You got it in {count} guesses.')


def welcome():
    print('Welcome to world\'s most mediocre guessing game!')
    print('I am thinking of a number from 1 to 10...')


def byebye():
    print('Thanks for playing!')


# ----------------------- REPL with simple graphics ------------------------

def draw_map_repl():
    """
    REPL version of draw_map from lab 3: Repeatedly asks user to enter text and
    then draws boxes corresponding to input as follows: 'w'/'a'/'s'/'d'
    indicate the next box to appear should be above/left/below/right of current
    box and 'r' indicates that the next box should appear at a random location.
    Other characters should have no effect. 'q' is used to exit the REPL. The
    first box to appear should be in the upper-left corner of the graphics
    window.
    """
    clear()
    x = 0
    y = 0
    while True:
        paint(x, y)
        action = input('what next? ')
        if action == 'r':
            x = randrange(COLUMNS)
            y = randrange(ROWS)
        elif action == 'a':
            x -= 1
        elif action == 'd':
            x += 1
        elif action == 'w':
            y -= 1
        elif action == 's':
            y += 1
        elif action == 'q':
            break
        else:
            print('invalid response')


# ----------------------- functions as values ------------------------

def square(z):
    u = z * z
    return u


# calls f twice in succession on x; shows how function can be an argument
def twice(f, x):
    y = f(f(x))
    return y


def is_digit(ch):
    b = len(ch) == 1 and '0' <= ch and ch <= '9'
    return b


def is_upper(ch):
    b = len(ch) == 1 and 'A' <= ch and ch <= 'Z'
    return b


# compare to has function from lab4 examples
def has_any(s, property):
    """
    Returns True if property (a function that takes as input a string and
    returns a Boolean value) is True for any character that occurs in string s,
    False otherwise. Returns False if s is empty.

    >>> has_any('C3P0', is_digit)
    True
    >>> has_any('0 is nowhere', is_upper)
    False
    """
    found = False
    i = 0
    while not found and i < len(s):
        found = property(s[i])
        i += 1
    return found
